﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using RecrownedAthenaeum.Render;

namespace RecrownedAthenaeum.UI.Modular
{

    /// <summary>
    /// Contains a group of modules and has its own relative coordinate system.
    /// </summary>
    public class UIModuleGroup : UIModule
    {
        List<UIModule> modules = new List<UIModule>();
        ScissorBox scissorBox;

        /// <summary>
        /// Configuration for starting sprite batch after scissoring. Only used if cropping.
        /// </summary>
        public SpriteBatchSettings spriteBatchSettings;

        /// <summary>
        /// Creates a module group.
        /// </summary>
        /// <param name="crop">Whether or not to crop out of bounds. Default is false.</param>
        public UIModuleGroup(bool crop = false)
        {
            this.spriteBatchSettings = Configuration.SpriteBatchSettings.Value;
            if (crop)
            {
                scissorBox = new ScissorBox();
            }
        }

        /// <summary>
        /// Draws this group of modules.
        /// </summary>
        /// <param name="batch">Batch used to draw the group.</param>
        public override void Draw(SpriteBatch batch)
        {
            if (scissorBox != null)
            {
                batch.End();
                scissorBox.Begin(Boundaries, batch, spriteBatchSettings);
            }

            foreach (UIModule module in modules)
            {
                int offsetX = module.situation.X;
                int offsetY = module.situation.Y;
                module.situation.X = situation.X + offsetX;
                module.situation.Y = situation.Y + offsetY;
                module.Draw(batch);
                module.situation.X = offsetX;
                module.situation.Y = offsetY;
            }

            if (scissorBox != null)
            {
                scissorBox.End();

            }
        }

        /// <summary>
        /// Updates the group of modules.
        /// </summary>
        /// <param name="gameTime">Game time used.</param>
        public override void Update(GameTime gameTime)
        {
            foreach (UIModule module in modules)
            {
                module.Update(gameTime);
            }
        }

        /// <summary>
        /// Adds module(s) to this group.
        /// </summary>
        /// <param name="addModules">The module(s) to add.</param>
        public void AddModule(params UIModule[] addModules)
        {
            foreach (UIModule module in addModules)
            {
                if (modules.Contains(module))
                {
                    throw new InvalidOperationException(module.ToString() + " already exists in " + this.ToString());
                }
                module.parent = this;
                modules.Add(module);
            }
        }

        /// <summary>
        /// Removes given module from group.
        /// </summary>
        /// <param name="module">module to remove.</param>
        public void RemoveModule(UIModule module)
        {
            module.parent = null;
            modules.Remove(module);
        }

        /// <summary>
        /// Updates the keyboard state of the modules in this group.
        /// </summary>
        /// <param name="state">The new state.</param>
        /// <returns>Whether or not to continue updating the other listeners.</returns>
        public override bool KeyboardStateChanged(KeyboardState state)
        {
            foreach (UIModule module in modules)
            {
                module.KeyboardStateChanged(state);
            }
            return base.KeyboardStateChanged(state);
        }

        /// <summary>
        /// Updates the moues state of the modules in this group.
        /// </summary>
        /// <param name="state">The new state.</param>
        /// <returns>Whether or not to continue updating other listeners.</returns>
        public override bool MouseStateChanged(MouseState state)
        {
            foreach (UIModule module in modules)
            {
                module.MouseStateChanged(state);
            }
            return base.MouseStateChanged(state);
        }
    }
}
